@@ -1,9 +1,8 @@
-import json
-from collections.abc import Collection, MutableMapping, MutableSequence
+from collections.abc import Collection
 from http import HTTPStatus
 from logging import Logger, getLogger
 from time import time
-from typing import Any, Literal, TypeAlias, cast
+from typing import Any, Literal
 
 from starlette import status
 from starlette.middleware.base import BaseHTTPMiddleware, DispatchFunction, RequestResponseEndpoint
@@ -13,22 +12,19 @@ from starlette.types import ASGIApp
 
 from py_fastapi_logging.config.config import init_logger
 from py_fastapi_logging.constants import REQUEST_ID_HEADER
+from py_fastapi_logging.middlewares.data_filter import DataFilter
 from py_fastapi_logging.middlewares.utils.request_id import generate_request_id
 from py_fastapi_logging.schemas.request import RequestPayload
 from py_fastapi_logging.schemas.response import ResponsePayload
 from py_fastapi_logging.utils.extra import set_extra_to_environ
 
-_Payload: TypeAlias = RequestPayload | ResponsePayload
-_Params: TypeAlias = dict[str, Any] | None
-
 
 class LoggingMiddleware(BaseHTTPMiddleware):
     _app_name: str
     _prefix: str | None
     _logger: Logger
     _exclude_requests_starting_with: frozenset[str]
-    _filtered_fields: frozenset[str]
-    _marker_filtered: str
+    _data_filter: DataFilter
 
     def __init__(
         self,
@@ -46,8 +42,7 @@ class LoggingMiddleware(BaseHTTPMiddleware):
         self._app_name = app_name
         self._prefix = prefix
         self._exclude_requests_starting_with = frozenset(exclude_requests_starting_with or ())
-        self._filtered_fields = frozenset(filtered_fields or ())
-        self._marker_filtered = marker_filtered
+        self._data_filter = DataFilter(filtered_fields or (), marker_filtered)
         if logger is None:
             init_logger(app_name)
             self._logger = getLogger("default")
@@ -109,10 +104,10 @@ class LoggingMiddleware(BaseHTTPMiddleware):
                 host=f"{host_or_socket}:{port}" if port is not None else host_or_socket,
                 body=request_body,
             )
-
+            self._data_filter.filter_request_payload(request_log_payload, request.headers)
             extra_payload: dict[str, Any] = common_extras | {
                 "tags": ["API", "REQUEST"],
-                "payload": self._filter_private_data(request_log_payload),
+                "payload": request_log_payload,
             }
             self._logger.info(f"REQUEST {request.method} {request.url.path}", extra=extra_payload)
 
@@ -149,9 +144,10 @@ class LoggingMiddleware(BaseHTTPMiddleware):
                 "response_time": f"{duration}ms",
                 "response_body": response_body.decode(),
             }
+            self._data_filter.filter_response_payload(response_log_payload)
             extra_payload = common_extras | {
                 "tags": ["API", "RESPONSE"],
-                "payload": self._filter_private_data(response_log_payload),
+                "payload": response_log_payload,
             }
             self._logger.info(f"RESPONSE {response.status_code} {request.url.path}", extra=extra_payload)
 
@@ -159,48 +155,3 @@ class LoggingMiddleware(BaseHTTPMiddleware):
 
     def _is_url_path_in_exclude_list(self, request: Request) -> bool:
         return any(request.url.path.startswith(path) for path in self._exclude_requests_starting_with)
-
-    def _filter_private_data(self, payload: _Payload) -> _Payload:
-        if not self._filtered_fields:
-            return payload
-
-        match payload:
-            case {"method": _, "path": _, "host": _, "params": params, "body": str(body)} as value:
-                payload = cast(RequestPayload, value)
-                payload["params"] = self._filter_params(cast(_Params, params))
-                payload["body"] = self._filter_body(body)
-            case {"status": _, "response_time": _, "response_body": str(body)} as value:
-                payload = cast(ResponsePayload, value)
-                payload["response_body"] = self._filter_body(body)
-
-        return payload
-
-    def _filter_params(self, params: _Params) -> _Params:
-        if not params:
-            return params
-
-        new_params: dict[str, Any] = params.copy()
-        for key in params:
-            if key in self._filtered_fields:
-                new_params[key] = self._marker_filtered
-
-        return new_params
-
-    def _filter_body(self, body: str) -> str:
-        try:
-            request_json: Any = json.loads(body)
-            response_json: Any = self._filter_json_body(request_json)
-            return json.dumps(response_json, ensure_ascii=False, separators=(",", ":"))
-        except Exception:
-            return body
-
-    def _filter_json_body(self, body: Any) -> Any:
-        if isinstance(body, MutableMapping):
-            return {
-                key: self._filter_json_body(value) if key not in self._filtered_fields else self._marker_filtered
-                for key, value in body.items()
-            }
-        elif isinstance(body, MutableSequence):
-            return list(map(self._filter_json_body, body))
-        else:
-            return body
